"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrokerInit = void 0;
const moleculer_1 = require("moleculer");
const debugServer = require('debug')('adonis:nats:server');
const debugClient = require('debug')('adonis:nats:client');
class BrokerInit {
    constructor(app) {
        this.app = app;
        this.isBrokerInit = false;
    }
    init(brokerOptions = {}) {
        const config = this.app.container.use('Adonis/Core/Config').get('nats', {});
        this.broker = new moleculer_1.ServiceBroker({ ...config, ...brokerOptions });
        return this;
    }
    addClient(clientClass, serverClass, brokerOptions = {}) {
        const cuid = this.app.helpers.cuid;
        const self = this;
        this.isBrokerInit = false;
        clientClass.prototype.start = async function () {
            if (!this.isBrokerInit) {
                debugClient('client broker first start');
                this.isBrokerInit = true;
                self.init({ nodeID: `${serverClass.serviceName}-client-${cuid()}`, ...brokerOptions });
                await self.broker.start();
            }
        };
        for (const method of Object.getOwnPropertyNames(serverClass.prototype)) {
            if (method !== 'constructor') {
                debugClient(`add method: ${method}`);
                clientClass.prototype[method] = async function (body = {}, options = {}) {
                    await this.start();
                    // const callName = `${serverClass.serviceName}.${method}`
                    // const logName = `${callName}`
                    // console.timelogName)
                    try {
                        const result = await self.broker.call(`${serverClass.serviceName}.${method}`, body, options);
                        debugClient(`result for method: ${result}`);
                        // console.timeEnd(logName)
                        return result;
                    }
                    catch (error) {
                        // console.timeEnd(logName)
                        throw error;
                    }
                };
            }
        }
        return new clientClass();
    }
    async addServer(classProto, brokerOptions = {}, serviceOptions = {}) {
        const cuid = this.app.helpers.cuid;
        this.init({ nodeID: `${classProto.serviceName}-${cuid()}`, ...brokerOptions });
        const actions = {};
        const classInstance = new classProto();
        for (const method of Object.getOwnPropertyNames(classProto.prototype)) {
            if (method !== 'constructor') {
                actions[method] = async (ctx) => {
                    debugServer(`handle action: ${classProto.serviceName}.${method}`);
                    return await classInstance[method](ctx.params);
                };
            }
        }
        debugServer(`add actions: ${classProto.serviceName}`, Object.keys(actions));
        this.broker.createService({
            ...(classProto?.serviceOptions || {}),
            ...(serviceOptions || {}),
            name: classProto.serviceName,
            actions,
        });
        await this.broker.start();
        return classInstance;
    }
}
exports.BrokerInit = BrokerInit;
